pub struct BindingFlags(/* private fields */);Expand description
Flags on a Binding.
Implementations
Sourceimpl BindingFlags
impl BindingFlags
Sourcepub const EXPLICIT_EXPORT: Self
pub const EXPLICIT_EXPORT: Self
The binding represents an explicit re-export.
For example, the binding could be FastAPI in:
from fastapi import FastAPI as FastAPISourcepub const EXTERNAL: Self
pub const EXTERNAL: Self
The binding represents an external symbol, like an import or a builtin.
For example, the binding could be FastAPI in:
from fastapi import FastAPISourcepub const ALIAS: Self
pub const ALIAS: Self
The binding is an aliased symbol.
For example, the binding could be app in:
from fastapi import FastAPI as appSourcepub const NONLOCAL: Self
pub const NONLOCAL: Self
The binding is nonlocal to the declaring scope. This could be a binding created by
a nonlocal statement, or a binding that shadows such a binding.
For example, both of the bindings in the following function are nonlocal:
def f():
nonlocal x
x = 1Sourcepub const GLOBAL: Self
pub const GLOBAL: Self
The binding is global. This could be a binding created by a global statement, or a
binding that shadows such a binding.
For example, both of the bindings in the following function are global:
def f():
global x
x = 1Sourcepub const DELETED: Self
pub const DELETED: Self
The binding was deleted (i.e., the target of a del statement).
For example, the binding could be x in:
del xThe semantic model will typically shadow a deleted binding via an additional binding
with BindingKind::Deletion; however, conditional deletions (e.g.,
if condition: del x) do not generate a shadow binding. This flag is thus used to
detect whether a binding was ever deleted, even conditionally.
Sourcepub const INVALID_ALL_FORMAT: Self
pub const INVALID_ALL_FORMAT: Self
The binding represents an export via __all__, but the assigned value uses an invalid
expression (i.e., a non-container type).
For example:
__all__ = 1Sourcepub const INVALID_ALL_OBJECT: Self
pub const INVALID_ALL_OBJECT: Self
The binding represents an export via __all__, but the assigned value contains an
invalid member (i.e., a non-string).
For example:
__all__ = [1]Sourcepub const PRIVATE_DECLARATION: Self
pub const PRIVATE_DECLARATION: Self
The binding represents a private declaration.
For example, the binding could be _T in:
_T = "This is a private variable"Sourcepub const UNPACKED_ASSIGNMENT: Self
pub const UNPACKED_ASSIGNMENT: Self
The binding represents an unpacked assignment.
For example, the binding could be x in:
(x, y) = 1, 2Sourcepub const IN_EXCEPT_HANDLER: Self
pub const IN_EXCEPT_HANDLER: Self
The binding took place inside an exception handling.
For example, the x binding in the following example
would not have this flag set, but the y binding would:
try:
x = 42
except RuntimeError:
y = 42Sourcepub const ANNOTATED_TYPE_ALIAS: Self
pub const ANNOTATED_TYPE_ALIAS: Self
The binding represents a PEP 613 explicit type alias.
Sourcepub const DEFERRED_TYPE_ALIAS: Self
pub const DEFERRED_TYPE_ALIAS: Self
The binding represents a PEP 695 type statement
Sourcepub const IN_ASSERT_STATEMENT: Self
pub const IN_ASSERT_STATEMENT: Self
The binding took place inside an assert statement
For example, x in the following snippet:
assert (x := y**2) > 42, xSourcepub const TYPE_ALIAS: Self
pub const TYPE_ALIAS: Self
The binding represents any type alias.
Sourceimpl BindingFlags
impl BindingFlags
Sourcepub const fn bits(&self) -> u16
pub const fn bits(&self) -> u16
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u16) -> Option<Self>
pub const fn from_bits(bits: u16) -> Option<Self>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u16) -> Self
pub const fn from_bits_truncate(bits: u16) -> Self
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u16) -> Self
pub const fn from_bits_retain(bits: u16) -> Self
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<Self>
pub fn from_name(name: &str) -> Option<Self>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: Self) -> bool
pub const fn intersects(&self, other: Self) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: Self) -> bool
pub const fn contains(&self, other: Self) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn remove(&mut self, other: Self)
pub fn remove(&mut self, other: Self)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: Self)
pub fn toggle(&mut self, other: Self)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: Self, value: bool)
pub fn set(&mut self, other: Self, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: Self) -> Self
pub const fn intersection(self, other: Self) -> Self
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(self, other: Self) -> Self
pub const fn union(self, other: Self) -> Self
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(self, other: Self) -> Self
pub const fn difference(self, other: Self) -> Self
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: Self) -> Self
pub const fn symmetric_difference(self, other: Self) -> Self
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> Self
pub const fn complement(self) -> Self
The bitwise negation (!) of the bits in self, truncating the result.
Sourceimpl BindingFlags
impl BindingFlags
Sourcepub const fn iter(&self) -> Iter<BindingFlags> ⓘ
pub const fn iter(&self) -> Iter<BindingFlags> ⓘ
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<BindingFlags> ⓘ
pub const fn iter_names(&self) -> IterNames<BindingFlags> ⓘ
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.