pub struct ScopeId(/* private fields */);Expand description
Id uniquely identifying a scope in a program.
Using a u32 is sufficient because Ruff only supports parsing documents with a size of max u32::max
and it is impossible to have more scopes than characters in the file (because defining a function or class
requires more than one character).
Implementations